@@ -40,6 +40,8 @@ static int ItclCreateMemberCode(Tcl_Interp* interp, ItclClass *iclsPtr,
 static int ItclCreateMemberFunc(Tcl_Interp* interp, ItclClass *iclsPtr,
 	Tcl_Obj *namePtr, const char* arglist, const char* body,
         ItclMemberFunc** imPtrPtr, int flags);
+static int Itcl_GetUplevelContext(Tcl_Interp *interp, ItclClass **iclsPtrPtr,
+       	ItclObject **ioPtrPtr, int level);
 
 
 /*
@@ -524,98 +526,100 @@ ItclCreateMemberFunc(
     if ((body != NULL) && (body[0] == '@')) {
         /* check for builtin cget isa and configure and mark them for
 	 * use of a different arglist "args" for TclOO !! */
+        imPtr->codePtr->flags |= ITCL_BUILTIN;
 	if (strcmp(name, "cget") == 0) {
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "configure") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "isa") == 0) {
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "createhull") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "keepcomponentoption") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
+	}
+	if (strcmp(name, "ignorecomponentoption") == 0) {
+	    imPtr->argcount = 0;
+	    imPtr->maxargcount = -1;
+	}
+	if (strcmp(name, "renamecomponentoption") == 0) {
+	    imPtr->argcount = 0;
+	    imPtr->maxargcount = -1;
+	}
+	if (strcmp(name, "addoptioncomponent") == 0) {
+	    imPtr->argcount = 0;
+	    imPtr->maxargcount = -1;
+	}
+	if (strcmp(name, "ignoreoptioncomponent") == 0) {
+	    imPtr->argcount = 0;
+	    imPtr->maxargcount = -1;
+	}
+	if (strcmp(name, "renameoptioncomponent") == 0) {
+	    imPtr->argcount = 0;
+	    imPtr->maxargcount = -1;
 	}
 	if (strcmp(name, "setupcomponent") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "itcl_initoptions") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "mytypemethod") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
             imPtr->flags |= ITCL_COMMON;
 	}
 	if (strcmp(name, "mymethod") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "mytypevar") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
             imPtr->flags |= ITCL_COMMON;
 	}
 	if (strcmp(name, "myvar") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "itcl_hull") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
             imPtr->flags |= ITCL_COMPONENT;
 	}
 	if (strcmp(name, "callinstance") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "getinstancevar") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "myproc") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
             imPtr->flags |= ITCL_COMMON;
 	}
 	if (strcmp(name, "installhull") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "destroy") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "installcomponent") == 0) {
 	    imPtr->argcount = 0;
 	    imPtr->maxargcount = -1;
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
 	}
 	if (strcmp(name, "info") == 0) {
-            imPtr->codePtr->flags |= ITCL_BUILTIN;
             imPtr->flags |= ITCL_COMMON;
 	}
     }
@@ -903,6 +907,21 @@ ItclCreateMemberCode(
 	    if (strcmp(body, "@itcl-builtin-keepcomponentoption") == 0) {
 	        isDone = 1;
 	    }
+	    if (strcmp(body, "@itcl-builtin-ignorecomponentoption") == 0) {
+	        isDone = 1;
+	    }
+	    if (strcmp(body, "@itcl-builtin-renamecomponentoption") == 0) {
+	        isDone = 1;
+	    }
+	    if (strcmp(body, "@itcl-builtin-addoptioncomponent") == 0) {
+	        isDone = 1;
+	    }
+	    if (strcmp(body, "@itcl-builtin-ignoreoptioncomponent") == 0) {
+	        isDone = 1;
+	    }
+	    if (strcmp(body, "@itcl-builtin-renameoptioncomponent") == 0) {
+	        isDone = 1;
+	    }
 	    if (strcmp(body, "@itcl-builtin-setupcomponent") == 0) {
 	        isDone = 1;
 	    }
@@ -1086,14 +1105,17 @@ Itcl_GetMemberCode(
      */
 
     if (!Itcl_IsMemberCodeImplemented(mcode)) {
-        result = Tcl_VarEval(interp, "::auto_load ",
-	        Tcl_GetString(imPtr->fullNamePtr), (char*)NULL);
+        Tcl_DString buf;
 
+        Tcl_DStringInit(&buf);
+        Tcl_DStringAppend(&buf, "::auto_load ", -1);
+        Tcl_DStringAppend(&buf, Tcl_GetString(imPtr->fullNamePtr), -1);
+        result = Tcl_EvalEx(interp, Tcl_DStringValue(&buf), -1, 0);
+        Tcl_DStringFree(&buf);
         if (result != TCL_OK) {
-            char msg[256];
-            sprintf(msg, "\n    (while autoloading code for \"%.100s\")",
-                Tcl_GetString(imPtr->fullNamePtr));
-            Tcl_AddErrorInfo(interp, msg);
+            Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
+                    "\n    (while autoloading code for \"%s\")",
+                    Tcl_GetString(imPtr->fullNamePtr)));
             return result;
         }
         Tcl_ResetResult(interp);  /* get rid of 1/0 status */
@@ -1134,8 +1156,15 @@ CallItclObjectCmd(
     ItclObject *ioPtr = data[1];
     int objc = PTR2INT(data[2]);
     Tcl_Obj **objv = data[3];
+    void * ptr;
+    ItclClass *contextIclsPtr = NULL;
+    ItclObject *contextIoPtr = NULL;
+    ItclObjectInfo *infoPtr;
     
-    ItclShowArgs(1, "CallObjectCmd", objc, objv);
+    ItclShowArgs(1, "CallItclObjectCmd", objc, objv);
+    if (ioPtr != NULL) {
+        ioPtr->hadConstructorError = 0;
+    }
     if (imPtr->flags & (ITCL_CONSTRUCTOR|ITCL_DESTRUCTOR)) {
         oPtr = ioPtr->oPtr;
     } else {
@@ -1145,7 +1174,35 @@ CallItclObjectCmd(
         result =  ItclObjectCmd(imPtr, interp, oPtr, imPtr->iclsPtr->clsPtr,
                 objc, objv);
     } else {
-        result = ItclObjectCmd(imPtr, interp, NULL, NULL, objc, objv);
+        ptr = Itcl_GetCallFrameVarFramePtr(interp);
+        if (Itcl_GetUplevelCallFrame(interp, 0) != ptr) {
+            /* we are executing an uplevel command (SF bug #244) */
+            if (ioPtr != NULL) {
+                if (imPtr->codePtr->flags & ITCL_BUILTIN) {
+                    /* it is a builtin command (SF bug #255 and # 256) */
+                    result = ItclObjectCmd(imPtr, interp, oPtr, NULL, objc, objv);
+                } else {
+                    infoPtr = (ItclObjectInfo *)Tcl_GetAssocData(interp,
+                            ITCL_INTERP_DATA, NULL);
+                    if (Itcl_GetStackSize(&infoPtr->contextStack) <= 1) {
+                        oPtr = ioPtr->oPtr;
+                        result = ItclObjectCmd(imPtr, interp, oPtr, NULL, objc, objv);
+                    } else {
+                        /* we are executing an uplevel command (SF bug #250) */
+                        if (Itcl_GetUplevelContext(interp, &contextIclsPtr, &contextIoPtr, -1) != TCL_OK) {
+                            return TCL_ERROR;
+                        }
+                        oPtr = contextIoPtr->oPtr;
+                        result =  ItclObjectCmd(imPtr, interp, oPtr, imPtr->iclsPtr->clsPtr,
+                                objc, objv);
+                    }
+                }
+            } else {
+                result = ItclObjectCmd(imPtr, interp, oPtr, NULL, objc, objv);
+	    }
+        } else {
+            result = ItclObjectCmd(imPtr, interp, oPtr, NULL, objc, objv);
+	}
     }
     if (result != TCL_OK) {
 	if (ioPtr != NULL && ioPtr->hadConstructorError == 0) {
@@ -1247,8 +1304,6 @@ Itcl_EvalMemberCode(
      */
     if (((mcode->flags & ITCL_IMPLEMENT_OBJCMD) != 0) ||
             ((mcode->flags & ITCL_IMPLEMENT_ARGCMD) != 0)) {
-	Tcl_Namespace *callerNsPtr;
-	callerNsPtr = Tcl_GetCurrentNamespace(interp);
 	Itcl_SetCallFrameNamespace(interp, imPtr->iclsPtr->nsPtr);
 
         if ((mcode->flags & ITCL_IMPLEMENT_OBJCMD) != 0) {
@@ -1438,6 +1493,79 @@ Itcl_GetContext(
     }
     return TCL_OK;
 }
+
+int
+Itcl_GetUplevelContext(
+    Tcl_Interp *interp,           /* current interpreter */
+    ItclClass **iclsPtrPtr,       /* returns:  class definition or NULL */
+    ItclObject **ioPtrPtr,
+    int level)        /* returns:  object data or NULL */
+{
+    Tcl_Namespace *activeNs = Tcl_GetCurrentNamespace(interp);
+    Tcl_HashEntry *hPtr;
+    ItclCallContext *callContextPtr;
+    ItclObjectInfo *infoPtr;
+
+    /*
+     *  Return NULL for anything that cannot be found.
+     */
+    *ioPtrPtr = NULL;
+
+    if (!Itcl_IsClassNamespace(activeNs)) {
+        /*
+         *  If there is no class/object context, return an error message.
+         */
+        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+            "namespace \"", activeNs->fullName, "\" is not a class namespace",
+            (char*)NULL);
+
+        return TCL_ERROR;
+    }
+    /*
+     *  If the active namespace is a class namespace, then return
+     *  all known info.  See if the current call frame is a known
+     *  object context, and if so, return that context.
+     */
+    infoPtr = (ItclObjectInfo *)Tcl_GetAssocData(interp,
+            ITCL_INTERP_DATA, NULL);
+    if (Itcl_GetStackSize(&infoPtr->contextStack)-1+level < 0 ||
+            level > Itcl_GetStackSize(&infoPtr->contextStack)) {
+        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+            "INTERNAL ERROR in Itcl_GetUplevelContext",
+            (char*)NULL);
+        return TCL_ERROR;
+    }
+    callContextPtr = Itcl_GetStackValue(&infoPtr->contextStack,
+            Itcl_GetStackSize(&infoPtr->contextStack)-1+level);
+    if ((callContextPtr != NULL) && (callContextPtr->imPtr != NULL)) {
+        *iclsPtrPtr = callContextPtr->imPtr->iclsPtr;
+    } else {
+        hPtr = Tcl_FindHashEntry(&infoPtr->namespaceClasses,
+                (char *)activeNs);
+        if (hPtr != NULL) {
+            *iclsPtrPtr = (ItclClass *)Tcl_GetHashValue(hPtr);
+        }
+    }
+    if (*iclsPtrPtr == NULL) {
+        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+            "namespace \"", activeNs->fullName, "\" is not a class namespace",
+            (char*)NULL);
+
+        return TCL_ERROR;
+    }
+
+    if (callContextPtr == NULL) {
+	/* must be a class namespace without an object */
+	*ioPtrPtr = NULL;
+	return TCL_OK;
+    }
+    *ioPtrPtr = callContextPtr->ioPtr;
+    if ((*ioPtrPtr == NULL) && ((*iclsPtrPtr)->nsPtr != NULL)) {
+        /* maybe we are in a constructor try currIoPtr */
+        *ioPtrPtr = (*iclsPtrPtr)->infoPtr->currIoPtr;
+    }
+    return TCL_OK;
+}
 
 /*
  * ------------------------------------------------------------------------
@@ -1469,7 +1597,6 @@ Itcl_GetMemberFuncUsage(
     ItclClass *iclsPtr;
     char *name;
     char *arglist;
-    int argcount;
 
     /*
      *  If the command is a method and an object context was
@@ -1526,14 +1653,11 @@ Itcl_GetMemberFuncUsage(
 	} else {
 	    arglist = NULL;
 	}
-        argcount = imPtr->argcount;
     } else {
         if (imPtr->argListPtr != NULL) {
             arglist = Tcl_GetString(imPtr->usagePtr);
-            argcount = imPtr->argcount;
         } else {
             arglist = NULL;
-            argcount = 0;
         }
     }
     if (arglist) {
@@ -2096,6 +2220,12 @@ Itcl_CmdAliasProc(
 	if (strcmp(cmdName, "@itcl-builtin-keepcomponentoption") == 0) {
 	    return Tcl_FindCommand(interp, "::itcl::builtin::keepcomponentoption", NULL, 0);
 	}
+	if (strcmp(cmdName, "@itcl-builtin-ignorecomponentoption") == 0) {
+	    return Tcl_FindCommand(interp, "::itcl::builtin::removecomponentoption", NULL, 0);
+	}
+	if (strcmp(cmdName, "@itcl-builtin-irgnorecomponentoption") == 0) {
+	    return Tcl_FindCommand(interp, "::itcl::builtin::ignorecomponentoption", NULL, 0);
+	}
 	if (strcmp(cmdName, "@itcl-builtin-setupcomponent") == 0) {
 	    return Tcl_FindCommand(interp, "::itcl::builtin::setupcomponent", NULL, 0);
 	}
@@ -2432,7 +2562,6 @@ ItclAfterCallMethod(
     Tcl_Namespace *nsPtr,
     int call_result)
 {
-    Tcl_Object oPtr;
     Tcl_HashEntry *hPtr;
     ItclObject *ioPtr;
     ItclMemberFunc *imPtr;
@@ -2440,7 +2569,6 @@ ItclAfterCallMethod(
     int newEntry;
     int result;
 
-    oPtr = NULL;
     imPtr = (ItclMemberFunc *)clientData;
 
     callContextPtr = NULL;
@@ -2658,8 +2786,7 @@ ItclProcErrorProc(
             Tcl_AppendToObj(objPtr, ")", -1);
         }
 
-        Tcl_AddErrorInfo(interp, Tcl_GetString(objPtr));
-        Tcl_DecrRefCount(objPtr);
+        Tcl_AppendObjToErrorInfo(interp, objPtr);
 	objPtr = NULL;
         loopCnt--;
     }
